@@ -51,7 +51,8 @@ CHAIN_ID = 'W'
 #MATCHER = 'https://nodes.wavesnodes.com'
 #MATCHER = 'http://matcher.wavesnodes.com'
 MATCHER = 'https://matcher.waves.exchange'
-MATCHER_PUBLICKEY = ''
+#MATCHER_PUBLICKEY = ''
+MATCHER_PUBLICKEY = '9cpfKN9suPNvfeUNphzxXMjcnn974eme8ZhWUjaktzU5'
 
 #DATAFEED = 'http://marketdata.wavesplatform.com'
 DATAFEED = 'https://api.wavesplatform.com'
